<?php

/*
* FothoFax - A Simple FiloFax replacement
* Copyright (C) 2005 Tom Fotherby
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 * Fothofax main code file
 * 
 * This file contains all Fothofax sourcecode logic minus the data editing page that is in fothoFaxEdit.php.
 * @author Tom Fotherby <fothofax@tomfotherby.com>
 * @version 0.41
 * @package Fothofax
 */

//////////////////////////////////////////////////////////////////////////
///////////  C o n f i g u r a t i o n    V a r i a b l e s //////////////
///////////                  O P T I O N A L                //////////////
//////////////////////////////////////////////////////////////////////////

  // Set this to the location of your address book data file relative to this file
  $bdayFile = 'AddressBook.txt';

  // Set this to the location of your calendar data file:
  $eventFile = 'EventsBook.txt';

  // Set this to the location of your notebook data file:
  $notebookFile = 'NoteBook.txt';

//////////////////////////////////////////////////////////////////////////
///////  D a t a     G a t h e r i n g     F u n c t i o n s  ////////////
///////        N O    E D I T I N G     N E E D E D           ////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
/**
 * The Fothofax data (e.g. addressbook) is either read from a file or a POST var. If it read from a POST var then it must be written to a file to save the changes made. 
 */
function setUpFothoFaxData($abaseDir =".")
{
  global $baseDir,$bdayFile,$eventFile,$notebookFile,$bdayArry,$ddayArry,$evntArry,$notebookLines,$bdayFileEditURL,$eventFileEditURL,$notebookFileEditURL;

  // Set the global base directory
  $baseDir = $abaseDir;

  // Blank this out if you DONT want users to be able to edit the birthday file (e.g. if it is an a wikiwiki page).
  $bdayFileEditURL = "<a href=\'$baseDir/fothoFaxEdit.php?mode=editAddressBook&parentPage=".$_SERVER['PHP_SELF']."\'><img border=0 src=\'$baseDir/FothoFaxImages/editBdays.gif\'>(Edit People)</a>"; 

  // Blank this out if you DONT want users to be able to edit the Events file (e.g. if it is an a wikiwiki page).
  $eventFileEditURL = "<a href=\'$baseDir/fothoFaxEdit.php?mode=editEventsBook&parentPage=".$_SERVER['PHP_SELF']."\'><img border=0 src=\'$baseDir/FothoFaxImages/editEvents.gif\'>(Edit Events)</a>"; 

  // Blank this out if you DONT want users to be able to edit the Notebook file (e.g. if it is an a wikiwiki page).
  $notebookFileEditURL = "<a href=\'$baseDir/fothoFaxEdit.php?mode=editNoteBook&parentPage=".$_SERVER['PHP_SELF']."\'><img border=0 src=\'$baseDir/FothoFaxImages/editNoteBook.gif\'>(Edit Notebook)</a>"; 

  //Gather all birthdays in an array:
  $bdayArry = gatherBdays($baseDir . "/". $bdayFile);
  //print_r($bdayArry);

  //Gather all deathdays in an array:
  $ddayArry = gatherDdays($bdayArry);
  //print_r($ddayArry);

  //Gather all events in an array:
  $evntArry = gatherEvents($baseDir . "/". $eventFile);
  //print_r($evntArry);

  //Gather notebook text:
  $notebookLines = gatherNoteBook($baseDir . "/". $notebookFile);
}

//////////////////////////////////////////////////////////////////////////
/**
 * This reads in a structured text file and returns an array of birthday data.
 * @param string $bdayFile name of file containing birthday data.
 * @return array Returns an array containing birthday data.
 */
function gatherBdays($bdayFile)
{
 $bdayLastSeen = "";   //birthday var
 $ddayLastSeen = "";   //deathday var
 $currentName = "";
 $nameLastSeen = "";

 // Read/Write File according to whether there is a "contents" POST var.
 if (isset($_REQUEST['page']) && $_REQUEST['page'] == "AddressBook" &&
     isset($_POST['contents']) &&
     (!isset($_REQUEST['mode']) || $_REQUEST['mode'] != "Cancel")) 
 {
	// Need to write the file (don't need to read the file because we have it posted in the form)
        //  NOTE: PHP directive "magic_quotes_gpc" is on by default, and it runs addslashes() on all POST data. Therefore we have to strip them off before saving.
	if (isset($_REQUEST['mode']) && $_REQUEST['mode'] == "Save")
		tomWriteFile("$bdayFile",stripslashes($_POST['contents']));

	$fileLines = split("\n", stripslashes($_POST['contents']));
 } else {
	// Read entire file from disk into an array
 	$fileLines = file("$bdayFile");
 }

 foreach ($fileLines as $line_num => $line) {

  if (preg_match("/^\s+\*\sName\:\s([\w \-]+)/",$line,$matches)) {
    $nameLastSeen = chop($matches[1]);  //chop removes trailing whitespace.
  }

  if (preg_match("/^\s+\*\sBirthday\:/",$line,$matches)) {
    if (preg_match("/^\s+\*\sBirthday\:\s([\d\w\/]+)/",$line,$matches)) {
      $bdayLastSeen = $matches[1];      // $bdayLastSeen will be in the format: "20/Mar/1980".

      // Check date format
      if (!preg_match("/^\d\d\/\w\w\w\/\d\d\d\d$/",$bdayLastSeen,$matches)) {
	print "FotherFax Problem! <b><font color=\"red\">Unknown date format</font></b> for birthday. Valid date format is \"DD/MMM/YYYY\". Please correct \"$bdayFile\" and re-load page. The line causing the problem is \"$line\".\n";
	$bdayLastSeen = "";
      }
    }
  }

  if (preg_match("/^\s+\*\sDeathday\:\s([\d\w\/]+)/",$line,$matches)) {
    $ddayLastSeen = $matches[1];      // $ddayLastSeen will be in the format: "20/Mar/1980".

      // Check date format
      if (!preg_match("/^\d\d\/\w\w\w\/\d\d\d\d$/",$ddayLastSeen,$matches)) {
	print "FotherFax Problem! <b><font color=\"red\">Unknown date format</font></b> for deathday. Valid date format is \"DD/MMM/YYYY\". Please correct \"$bdayFile\" and re-load page. The line causing the problem is \"$line\".\n";
	$bdayLastSeen = "";
      }
  }

  //After going through one name and getting to the next, print out relevant info
  if ($nameLastSeen != $currentName || $line_num == sizeof($fileLines)-1) {

    if ($bdayLastSeen != "") {
      $pplArray["$currentName"] = array("Name" => $currentName,
                                        "Bday" => $bdayLastSeen,
                                        "Age"  => strlen($bdayLastSeen) > 6 ? (Date("Y") - substr($bdayLastSeen,7,4)) : "?");

      if ($ddayLastSeen != "")
      	$pplArray["$currentName"]["Dday"] = $ddayLastSeen; //Add a deathday for this person

      $bdayArry[substr($bdayLastSeen,0,6)][] = $pplArray["$currentName"];
    }

    $currentName = $nameLastSeen;
    $bdayLastSeen = "";
    $ddayLastSeen = "";
  }
 }
  return $bdayArry;
}

//////////////////////////////////////////////////////////////////////////
/**
 * This creates an array of deathDays given an array of people information
 * @param string $bdayArry Array containing birthday and deathday data.
 * @return array Returns an array containing deatday data.
 */
function gatherDdays($bdayArry)
{
 foreach ($bdayArry as $personArray) {
  foreach ($personArray as $person) {
   if ($person["Dday"] != "")
      $oDdayArray[substr($person["Dday"],0,6)][] = $person;
  }
 }
 return $oDdayArray;
}

//////////////////////////////////////////////////////////////////////////
/**
 * This gathers the text of a file ment as a kind of "notebook".
 * @param string $noteBookFile name of file containing notebook data.
 * @return integer Returns a count of the number of lines in the notebook.
 */
function gatherNoteBook($noteBookFile)
{
 // Read/Write File according to whether there is a "contents" POST var.
 if (isset($_REQUEST['page']) && $_REQUEST['page'] == "NoteBook" &&
     isset($_POST['contents']) &&
     (!isset($_REQUEST['mode']) || $_REQUEST['mode'] != "Cancel")) 
 {
	// Need to write the file (don't need to read the file because it has been posted in the form)
	if (isset($_REQUEST['mode']) && $_REQUEST['mode'] == "Save")
		tomWriteFile("$noteBookFile",stripslashes($_POST['contents']));
	$fileLines = count(explode("\n", stripslashes($_POST['contents'])));
 } else {
	// Read the file from Disk and count the number of lines
 	$fileLines = count(file($noteBookFile));
 }
 return $fileLines;
}


//////////////////////////////////////////////////////////////////////////
/**
 * This reads in a file formatted in a particular way and outputs an array of events
 * This is an example of the input:
 * * Date: 10/Jun/2006
 *  * Anniversary[100]: Kix and my wedding Anniversary
 * * Date: 23-26/May/08
 *  * Event: Kix to Barcelona
 * * Date: 24/May/08-24/May/09
 *  * Event[pink]: Kix working in Barcelona
 * This is an example of the return array:
 *  $evntArry["20/Mar/2004"][0] = "Tom going for birthday meal";
 *  $evntArry["20/Mar/2004"][1] = "Tom opening birthday presents";
 *  $evntArry["26/Mar"][0] = "Mothers day;
 *  ...
 * @param string $eventFile name of file containing event data.
 */
function gatherEvents($eventFile)
{
 $dateLastSeen = "";
 $months = array('Jan'=>'1','Feb'=>'2','Mar'=>'3','Apr'=>'4','May'=>'5','Jun'=>'6', 'Jul'=>'7', 'Aug'=>'8', 'Sep'=>'9', 'Oct'=>'10', 'Nov'=>'11', 'Dec'=>'12');

 // Read/Write File according to whether there is a "contents" POST var.
 if (isset($_REQUEST['page']) && $_REQUEST['page'] == "EventsBook" &&
     isset($_POST['contents']) &&
     (!isset($_REQUEST['mode']) || $_REQUEST['mode'] != "Cancel")) 
 {
        // We don't need to read the file because it has been posted in the form
	// In fact, we need to write the file

        //  NOTE: PHP directive "magic_quotes_gpc" is on by default, and it runs addslashes() on all POST data. Therefore we have to strip them off before saving.
	if (isset($_REQUEST['mode']) && $_REQUEST['mode'] == "Save")
	   tomWriteFile("$eventFile",stripslashes($_POST['contents']));
	$fileLines = split("\n", stripslashes($_POST['contents']));
 } else {
	// Read the file from Disk
 	$fileLines = file($eventFile);
 }

 foreach ($fileLines as $line_num => $line)
 {
  // If this line is a new date, parse it...
  if (preg_match("/^\s+\*\sDate\:\s([\-\d\w\/]+)/",$line,$matches)) {
    $annualEvent  = "";
    $foundDate = $matches[1];
    if (preg_match("/^(\d\d)\/(\w\w\w)\/(\d{2,4})$/",$foundDate,$dateCheck)) {
      // Single day event
      $eventStartJD = gregoriantojd($months[$dateCheck[2]],$dateCheck[1],get4LetterYear($dateCheck[3]));
      $eventEndJD   = $eventStartJD;
    } else if (preg_match("/^(\d\d)\/(\w\w\w)$/",$foundDate,$dateCheck)) {
      // Event doesn't have year (i.e. repeats annually).
      $annualEvent = $dateCheck[1] . "/" . $dateCheck[2];
    } else if (preg_match("/^(\d\d)\-(\d\d)\/(\w\w\w)\/(\d{2,4})$/",$foundDate,$dateCheck)) {
      // Event spans days of the same month
      $eventStartJD = gregoriantojd($months[$dateCheck[3]],$dateCheck[1],get4LetterYear($dateCheck[4]));
      $eventEndJD   = gregoriantojd($months[$dateCheck[3]],$dateCheck[2],get4LetterYear($dateCheck[4]));
    } else if (preg_match("/^(\d\d)\/(\w\w\w)\-(\d\d)\/(\w\w\w)\/(\d{2,4})$/",$foundDate,$dateCheck)) {
      // Event spans days over months
      $eventStartJD = gregoriantojd($months[$dateCheck[2]],$dateCheck[1],get4LetterYear($dateCheck[5]));
      $eventEndJD   = gregoriantojd($months[$dateCheck[4]],$dateCheck[3],get4LetterYear($dateCheck[5]));
    } else if (preg_match("/^(\d\d)\/(\w\w\w)\/(\d{2,4})\-(\d\d)\/(\w\w\w)\/(\d{2,4})$/",$foundDate,$dateCheck)) {
      // Event spans days over years
      $eventStartJD = gregoriantojd($months[$dateCheck[2]],$dateCheck[1],get4LetterYear($dateCheck[3]));
      $eventEndJD   = gregoriantojd($months[$dateCheck[5]],$dateCheck[4],get4LetterYear($dateCheck[6]));
    } else {
      print "FotherFax Problem! <b><font color=\"red\">Unknown date format</font></b> for line: \"$line\". Valid date format is \"DD/MMM\" or \"DD/MMM/YYYY\". Please correct \"$eventFile\" and re-load page.\n";
      exit;
    }
  }

  if (preg_match("/^\s+\*\sEvent\[?(\w*)\]?\:\s+(.*)/",$line,$matches))
  {
    $eventColour = $matches[1];
    $event       = chop($matches[2]); // remove trailing newline and whitespace

    if ($annualEvent != "")
    {
       // Special case of a annually repeating event (date without a year)
       $evntArry[$annualEvent][] = array("Details" => $event,
                                         "Colour"  => $eventColour,
                                         "Year"    => "",
					 "LineNum" => $line_num);
    } else {
       // Loop though the days that this event spans:
       for ($thisJD=$eventStartJD; $thisJD<=$eventEndJD; $thisJD++)
       {
          $thisGregorianDate = cal_from_jd($thisJD,CAL_GREGORIAN);
          $thisDay = prependZero($thisGregorianDate["day"]) . "/" . $thisGregorianDate["abbrevmonth"];

          $evntArry[$thisDay][] = array("Details" => $event,
                                        "Colour"  => $eventColour,
                                        "Year"    => $thisGregorianDate["year"],
					"LineNum" => $line_num,
					"StartJD" => $eventStartJD,
					"EndJD"   => $eventEndJD);
       }
    }
  } 
  else if (preg_match("/^\s+\*\sAnniversary\[?(\d*)\]?\:\s(.*)/",$line,$matches)) 
  {
    $repeatDayStep = $matches[1];  //Optional, therefore often blank.
    $event         = chop($matches[2]); // remove trailing newline and whitespace

    if ($annualEvent != "") {
      print "FotherFax Problem! <b><font color=\"red\">Anniversaries must have a full date including a year. Please correct \"$eventFile\" and re-load page.<\/font> Problem date: \"$eventDay/$eventMon/$eventYer\".\n";
      exit;
    }

    // Loop though the days that this anniversary spans:
    for ($thisJD=$eventStartJD; $thisJD<=$eventEndJD; $thisJD++)
    {
          $thisGregorianDate = cal_from_jd($thisJD,CAL_GREGORIAN);
          $thisDay = prependZero($thisGregorianDate["day"]) . "/" . $thisGregorianDate["abbrevmonth"];

          $evntArry[$thisDay][] = array("Details"     => $event,
                                        "Year"        => $thisGregorianDate["year"],
                                        "Anniversary" => 1,
                                        "RepeatStep"  => $repeatDayStep,
                                        "JulianDay"   => $thisJD,
					"LineNum" => $line_num);
    }

  }
 }
  return $evntArry;
}

//////////////////////////////////////////////////////////////////////////
/**
 * This outputs HTML that displays the calendar. It is expected that the setupFothoFaxData() function is called before this one to populate the global arrays with data.
 */
function drawCalendar()
{
  // Calculate how many weeks back the calendar should go (based on input params).
  if (!isset($_REQUEST['back']) || $_REQUEST['back'] < -99 || $_REQUEST['back'] > 99)
	$numWeeksBack = -1;
  else 
	$numWeeksBack = $_REQUEST['back'];

  // Calculate how many weeks forwards the calendar should go (based on input params).
  if (!isset($_REQUEST['forw']) || $_REQUEST['forw'] < -99 || $_REQUEST['forw'] > 99)
	$numWeeksForward = 24;
  else 
	$numWeeksForward = $_REQUEST['forw'];

  //Calculate how many days from today we start the calendar (based on input params)
  if (Date("w") == 0)   // If it is Sunday.
   $when2Start = (7*$numWeeksBack) - 6;
  else
   $when2Start = (7*$numWeeksBack) -(Date("w")-1);  //days since monday minus a week.

  //How many days do we want the calendar to show?
  $when2End = $when2Start + (7*$numWeeksForward) - (7*$numWeeksBack);

  $numOfWeekend = (-$numWeeksBack+$numWeeksForward);

  //Output the calendar title bar (i.e. 1st row in calendar table).
  echo "<table class=\"main\" cellpadding=\"0\" border=\"1\" cellspacing=\"0\">\n";
  echo "<tr>\n";
  echo  "<td class=\"title\">" . getCaptionBoxInfo() . "</td>\n";
  echo  "<td colspan=\"7\" class=\"title\">";
  echo  "<div id=\"dateRange\"><a class=\"title\" href=\"javascript:getNewDateRange(0,'$numOfWeekend week calendar');\">$numOfWeekend week calendar</a></div></td>\n";
  echo "</tr>\n";

  //Output the row containing the days of the week (i.e. 2nd row of calendar table).
  echo "<tr>\n<th class=\"daysofweek\">&nbsp;</th><th class=\"daysofweek\">M</th><th class=\"daysofweek\">T</th><th class=\"daysofweek\">W</th><th class=\"daysofweek\">T</th><th class=\"daysofweek\">F</th><th class=\"daysofweek\">S</th><th class=\"daysofweek\">S</th>\n</tr>\n";

  $dayCount = 0;

  //Loop through from the start day to the end day showing date for each day
  for ($i=$when2Start; $i<$when2End; $i++)
  {
    $curDay = strtotime($i." days");
    $cssText = "";

     if (strftime("%d",$curDay) <= 7)
	$cssText = "OnNewMonth";
     else if (strftime("%d",$curDay) == date("t",$curDay))
	$cssText = "OnLastDayOfMonth";

    if ($dayCount%7 == 0) //If Start of week -> Output the month name in col 1
      echo "<tr>\n<td class=\"month$cssText\">" . strftime("%b",$curDay) . "</td>\n";

    if ($i == 0)
     $cssText = "today$cssText";
    else if ($dayCount%7 >= 5)
     $cssText = "weekend$cssText";
    else
     $cssText = "weekday$cssText";

    # Get info for this day in hash form
    $dayInfo = getDayInfo($curDay,$cssText);

    if ($dayInfo["info"] == "0")
      echo "<td class=\"$cssText\">". $dayInfo["daySimple"] . "</td>\n";
    else
      //We use multiple css classes to change the style AND colour of the table item.
      echo "<td class=\"$cssText ".$dayInfo["colour"]."\">". $dayInfo["dayComplex"] . "</td>\n";

    if ($dayCount%7 == 6)   //End of week.
      echo "</tr>\n";

    $dayCount++;
  }

  echo "</table>\n";
}

//////////////////////////////////////////////////////////////////////////
/**
 * @return string Returns a string containing HTML for the calendar help box
 */
function getCaptionBoxInfo()
{
  global $bdayFileEditURL,$eventFileEditURL,$notebookFileEditURL,$bdayArry,$evntArry,$notebookLines;

  return "<a class=\"title\" href=\"javascript:void(0);\" onmouseover=\"return overlib('<ul><li>Version: <a href=\\'http://www.tomfotherby.com/Websites/FothoFax/\\'>0.41 (Oct 2008)</a><li>Author: <a href=\'http://www.tomfotherby.com\'>Tom Fotherby</a><li>Software License: <a href=\'http://www.gnu.org/copyleft/gpl.html\'>GPL</a><li>$bdayFileEditURL. Number: ". sizeof($bdayArry) ."<li>$eventFileEditURL. Number: ". sizeof($evntArry) ."<li>$notebookFileEditURL, Lines: $notebookLines</ul>',CAPTION,'About FothoFax',STICKY);\" onmouseout=\"return nd();\">?</a>";
}


//////////////////////////////////////////////////////////////////////////
/**
 * This function gathers all info about this day and returns HTML that describes it.
 * @param string $dateToQuery Timestamp for current day.
 * @param string $cssText variable that sets the CSS class for this day so it can be styled nicely.
 * @return string Returns a string containing HTML describing the day. e.g. birthday and events for that day.
 */
function getDayInfo($dateToQuery,$cssText)
{
  global $baseDir,$bdayArry,$ddayArry,$evntArry;

 $addedBdayInfo  = "";
 $addedEventInfo = "";
 $addedRemembranceInfo = "";

 // Set up return hash. By default, nothing is happening on this day
 $dayInfoHash["daySimple"] = strftime("%d",$dateToQuery);
 $dayInfoHash["info"]      = 0;
 $dayInfoHash["colour"]    = "";

 $dateToTest      = strftime("%d/%b", $dateToQuery);
 $date_MonDayYear = explode("/",strftime("%m/%d/%Y", $dateToQuery));
 $today_Mon       = $date_MonDayYear[0];
 $today_Day       = $date_MonDayYear[1];
 $today_Year      = $date_MonDayYear[2];
 $today_jd        = gregoriantojd($today_Mon,$today_Day,$today_Year);

 // See if there's any birthdays on this day
 if ($bdayArry[$dateToTest]) {
    foreach ($bdayArry[$dateToTest] as $person) {
       $currentName = $person["Name"];
       $age = $person["Age"];
       if ($person["Dday"] == "")
	$addedBdayInfo .= "<li>$currentName\'s $age". getSuffix($age) . " birthday";
       else
        $addedBdayInfo .= "<li>It would have been $currentName\'s $age". getSuffix($age) . " birthday";
    }
 }

 //Make sure leap babies get a birthday every year!
 if (!date("L") and               // If it's not leap year
     $dateToTest == "28/Feb" and  // and it's the day before the usual leap day
     $bdayArry["29/Feb"]) {       // and there are people with birthdays on the 29th.
    foreach ($bdayArry["29/Feb"] as $person) {
       $currentName = $person["Name"];
       $age = $person["Age"];
       if ($person["Dday"] == "")
	 $addedBdayInfo .= "<li>$currentName\'s $age". getSuffix($age) . " birthday";
       else
         $addedBdayInfo .= "<li>It would have been $currentName\'s $age". getSuffix($age) . " birthday";
       $addedBdayInfo .= " (because it\'s not a leap year this year)";
    }
 }

 // See if there's any remembrance days on this day
 if ($ddayArry[$dateToTest]) {
    foreach ($ddayArry[$dateToTest] as $person) {	
       $addedRemembranceInfo .= "<li>".$person["Name"]." died on " . $person["Dday"];
    }
 }

 // See if there's any events on this day
 if ($evntArry[$dateToTest]) {
    foreach ($evntArry[$dateToTest] as $event) {
       if ($event["Year"] == $today_Year || $event["Year"] == "" || isset($event["Anniversary"])) {
         $addedEventInfo .= "<li>" . makeStringSafe($event["Details"]);
 	 $dayInfoHash["colour"] = $event["Colour"];
      }
    }
 }

 // See if there's any anniversaries on this day
 foreach ($evntArry as $eventArray) {
  foreach ($eventArray as $event) {
    if ($event["RepeatStep"] != "" && ($today_jd-$event["JulianDay"])%$event["RepeatStep"] == 0)
     $addedEventInfo .= "<li>" . makeStringSafe("It was " . ($today_jd-$event["JulianDay"]) . " days since " . $event["Details"]);
  }
 }

 // Now that we have collected info for this day. Format how it is displayed:

 $popupText = "";  // var to store the text displayed in the popup box for this day
 $iconFile  = "";  // The name of the icon image file for the day

 if ($addedBdayInfo != "")
 {
	$popupText .= "<b>Birthdays</b><br><ul>$addedBdayInfo</ul>";
	$iconFile  .= "candle";
 }

 if ($addedRemembranceInfo != "")
 {
	$popupText .= "<b>Remembrances</b><br><ul>$addedRemembranceInfo</ul>";
	$iconFile  .= "poppy";
 }

 if ($addedEventInfo != "")
 {
	$popupText .= "<b>Events</b><br><ul>$addedEventInfo</ul>";
 }

 if ($popupText != "")
 {
     // This day has something going on in it
    $dayInfoHash["info"] = 1;

    // Is an icon used on this day? (i.e. a candle or poppy)
    $imgTxt = $iconFile == "" ? "" : "<img border=\"0\" alt=\"$iconFile.gif\" src=\"$baseDir/FothoFaxImages/$iconFile.gif\">";

    // Is there no event this day? If so, make sure the day number hyperlink isn't in bold
    $linkClass = ($addedEventInfo == "") ? "class=\"notBold\"" : "class=\"$cssText\"";

    $dayInfoHash["dayComplex"] = "<a $linkClass href=\"javascript:void(0);\" onmouseover=\"return overlib('$popupText',CAPTION,'On $dateToTest:');\" onmouseout=\"return nd();\">" . strftime("%d",$dateToQuery) . "$imgTxt</a>";
 }

 return $dayInfoHash;
}

//////////////////////////////////////////////////////////////////////////
/**
 * This prints out the next n events
 * @param integer $num The number of events to display.

 */
function OutputEvents($num)
{
  global $evntArry;

  $eventCount    = 0;
  $numDaysAhead  = 0;
  $curTime       = strtotime("today");
  $lastEventTime = getLastEventTime();
  $eventsSeen    = array();

  //Output upcoming events (Start from Today and work forwards one day at a time):
  while ($eventCount < $num and ($lastEventTime = -1 or $curTime <= $lastEventTime))
  {
    $curTime    = strtotime("+" . $numDaysAhead . " days");
    $curYear    = strftime ("%Y", $curTime);
    $dateToTest = strftime ("%d/%b", $curTime);  //e.g. "20/Mar"

    // See if there's any events on this day
    if ($evntArry[$dateToTest]) {
      foreach ($evntArry[$dateToTest] as $event) {
        if (!in_array($event["LineNum"],$eventsSeen) && ($event["Year"] == $curYear || $event["Year"] == "" || isset($event["Anniversary"]))) {
	   if ($numDaysAhead == 0)
	      $timeText = "<font color=\"red\"><b>today</b></font>";
	   else if ($numDaysAhead == 1)
	      $timeText = "<font color=\"red\"><b>tomorrow</b></font>";
	   else
	      $timeText = "in " . $numDaysAhead . " days";

           if (isset($event["StartJD"]) && isset($event["EndJD"]) && $event["StartJD"] != $event["EndJD"]) {
              $startDayGregorian = cal_from_jd($event["StartJD"],CAL_GREGORIAN);
              $endDayGregorian   = cal_from_jd($event["EndJD"],CAL_GREGORIAN);
	      $event["Details"] = $event["Details"] . " (".$startDayGregorian["day"] . "/" . $startDayGregorian["abbrevmonth"] ."-".$endDayGregorian["day"] . "/" . $endDayGregorian["abbrevmonth"].")";
	    }

      	   print "$dateToTest ($timeText): ".$event["Details"]."<br>\n";
           array_push($eventsSeen,$event["LineNum"]);
	   $eventCount++;
	}
      }
    }
    $numDaysAhead++;
    if ($numDaysAhead >= 365 && $lastEventTime = -1) { 
       // We can break because there are recurring events but we only print out the first occurence this year. 
       //print "<font color=\"red\"><b>Warning</b></font>, exiting from a potential infinite loop. 999 days ahead reached!\n";
       break;
    }
 }
}

//////////////////////////////////////////////////////////////////////////
/**
 * This prints out the next n birthdays
 * @param integer $num The number of birthdays to display.

 */
function OutputBirthdays($num)
{
  global $bdayArry;

  $bdayCount = 0;
  $numDaysAhead = 0;

  //Output upcoming birthdays (Start from Today and work forwards a year):
  while ($bdayCount < $num and $numDaysAhead < 365)
  {
     $dateToTest = strftime ("%d/%b", strtotime("+" . $numDaysAhead . " days"));
     // See if there's any birthdays on this day
     if ($bdayArry[$dateToTest]) {
    	 foreach ($bdayArry[$dateToTest] as $person) {
	   if ($numDaysAhead == 0)
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday <font color=\"red\"><b>today</b></font>. They are "  . $person["Age"]  . ".<br>\n";
	   else if ($numDaysAhead == 1)
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday <font color=\"red\"><b>tomorrow</b></font>. They are "  . $person["Age"]  . ".<br>\n";
	   else
	     print "$dateToTest: It's " . $person["Name"] . "'s birthday in $numDaysAhead days. They are "  . $person["Age"]  . ".<br>\n";
           $bdayCount++;
     	 }
  	}
	$numDaysAhead++;
  }
}



//////////////////////////////////////////////////////////////////////////
////////////////  U t i l i t y   F u n c t i o n s  /////////////////////
//////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////
/**
 * This loops through the eventArray Keys looking for the date furthest in the future.
 * @return string The date of the last event or -1 if there are reoccuring events. 
 */
function getLastEventTime()
{
  global $evntArry;

  $months = array('Jan'=>'1','Feb'=>'2','Mar'=>'3','Apr'=>'4','May'=>'5','Jun'=>'6', 'Jul'=>'7', 'Aug'=>'8', 'Sep'=>'9', 'Oct'=>'10', 'Nov'=>'11', 'Dec'=>'12');
  $lastDate = strtotime("today");

  foreach ($evntArry as $eventDate => $eventArray)
  {
    $eventDay   = substr($eventDate,0,2);
    $eventMonth = $months[substr($eventDate,3,3)];

    foreach ($evntArry[$eventDate] as $event) {
     if (!$event["Year"] || isset($event["Anniversary"])) {
       // No year means we have recuring events and there's no such thing as the last one.
       return -1;
     } else {
       $eventYear = $event["Year"];
       $curDate = mktime(0,0,0,$eventMonth,$eventDay,$eventYear);

       if ($curDate > $lastDate) {
         $lastDate = $curDate;
         //print "<b>Last</b>: $eventDay/$eventMonth/$eventYear<br>";
       }
     }
   }
  }
  return $lastDate;
}

//////////////////////////////////////////////////////////////////////////
/**
 * This utility function takes out any characters that causes Overlib to break. Overlib is the javascript popup library (written by Erik Bosrup) that I use.
 * @param string $theString The string we want to make safe for Overlib.
 * @return string The string that is safe for Overlib.
 */
function makeStringSafe($theString)
{
  $theString = str_replace("'", "\'", $theString);      //Escape single quote (but can't use html!)
  $theString = str_replace("\"", "&quot;", $theString); //Replace double-quotes with html.
  $theString = str_replace("\n", "", $theString); //Get rid of newlines.

  $theString = trim($theString); // Remove trailing newline chars.
  return $theString;
}

//////////////////////////////////////////////////////////////////////////
/**
 * Calculates a suffix for a numver. Example input is "22". Output would be "nd".
 * @param string $number A number we want the suffix for.
 * @return string A 2 letter suffix.
 */
function getSuffix($number)
{
   switch ( $number % 10 )
   {
     case '1': return 'st';
     case '2': return 'nd';
     case '3': return 'rd';
     default:  return 'th';
   }
}

//////////////////////////////////////////////////////////////////////////
/**
 * Get the full year. 08 -> 2008
 * @param string $year The two digit year
 * @return string The 4 digit year.
 */
function get4LetterYear($year)
{
   if (strlen($year) == 4)
      return $year;
   else if (strlen($year) == 2)
      return substr(strftime("%Y", strtotime("today")),0,2) . $year;
   else
      print "FotherFax Problem! bad year input to get4LetterYear(\"$year\")\n";
}

//////////////////////////////////////////////////////////////////////////
/**
 * If a number is less than 10, prepend "0".
 * @param string $day The day number
 * @return string The 2 digit day number.
 */
function prependZero($day)
{
   if ($day < 10) return "0".$day;
   return $day;
}

//////////////////////////////////////////////////////////////////////////
/**
 * Read a file from disk
 * @param string $filename The filename of the file to be read.
 * @return string Returns the contents of the file.
 */
function tomReadFile( $filename )
{
        $result = "";                   
        $size = filesize($filename);
        if ( $size>0 )
        {
                $handle = fopen($filename, "r");
                $result = fread($handle, $size);
                fclose($handle);                                        
        }
        return $result;
}

//////////////////////////////////////////////////////////////////////////
/**
 * Writes a file to disk
 * @param string $title The filename of the file to be written.
 * @param string $contents The contents to be written to the file.
 * @return integer Returns 1 for success and 0 for failure.
 */
function tomWriteFile( $title, $contents )
{
	if (!$fd = @fopen($title, "w" ))
	{ 
		echo("Cannot open server's file for writing: $title");
		return 1;
	}
	
	if (@fwrite( $fd, $contents ) === FALSE)
	{
		echo("Cannot write to server's file: $title");
		return 2;
	}
	
	fclose( $fd );	
	return 0;	
}

?>